/*

   FLEXCOPY - program to copy one disk to another, across formats.

   By Moshe Braner  <braner@amvax.tn.cornell.edu>          870108


   FUNCTION

   This program copies the complete contents of one disk onto another.
   It first finds out which sector is the last one actually holding data,
   and copies all sectors up to that one.  The destination disk has to
   have enough space to receive all the data, and may be preformatted,
   or formatted through this program. The source or destination may also
   be an installed RAM disk.  One disk may have a different PHYSICAL
   structure than the other (e.g. a different number of sides), but both
   must share the same LOGICAL structure (i.e. the same size sectors,
   clusters, FAT and root directory).

   NOTE

   You CAN copy a double-sided disk to a single-sided one, provided
   the consecutive block of sectors holding all the data fits on the
   smaller disk.  That may not be the case, even when TOS tells you that
   less than 351K are used, since there may be empty "holes" in the
   middle of the used sectors, where files have been deleted.  To get
   around that, copy the data FILE BY FILE to a virgin RAM disk, then
   use FLEXCOPY to copy the RAM disk to the destination floppy disk.

   WARNINGS

   Not for hard disks (or future, very large, floppies).
   (12-bit FAT entries assumed.)
   This program will not work with "copy protected" disk formats,
   including the "FAST" format with its "dead" sectors.
*/

#include <osbind.h>

#define SECSIZE	512	/* >>>> for now 512-byte sectors only */

#define WORD	int	/* 16 bits: 'int' in Megamax */

#define OK	0
#define ERROR	-1
#define READ	0
#define WRITE	1
#define RREAD	2
#define WWRITE	3

typedef struct bpb {
	WORD	recsiz;
	WORD	clsiz;
	WORD	clsizb;
	WORD	rdlen;
	WORD	fsiz;
	WORD	fatrec;
	WORD	datrec;
	WORD	numcl;
	WORD	bflags;
} BPB;

error(msg)
	char msg[];
{
	Cconws(msg);
	Cconws("\007\r\n\n\tHit any key ");
	Cconin();
	exit(0);
}

int
format(drive, sides)
	int	drive, sides;
{
	register int	i, j, ds;
	long	a, b;
	static	WORD	buf[5000];
	int	virgin = 0xE5E5;
	long	magic = 0x87654321;

	if (drive >= 'A' && drive <= 'p')
		drive -= 'A';
	if (drive >= 'a' && drive <= 'p')
		drive -= 'a';
	if (sides=='d' || sides=='D' || sides=='2' || sides==1)
		ds = 1;
	else
		ds = 0;
	for (i=0; i<80; i++) {
	    for (j=0; j<=ds; j++) {
	        if (Flopfmt(buf,0L,drive,9,i,j,1,magic,virgin)!=OK)
	            return (ERROR);
	    }
	}
	for (i=0; i<9*256; i++)		/* one (one-sided) track */
		buf[i] = 0;
	if (Flopwr(buf,0L,drive,1,0,0,9)!=OK)	/* Zero 18 sectors */
		return (ERROR);
	if (Flopwr(buf,0L,drive,1,1-ds,ds,9)!=OK)
		return (ERROR);
	Protobt(buf, 0x01000000L, 2+ds, 0);	/* prototype boot block */
	if (Flopwr(buf,0L,drive,1,0,0,1)!=OK)	/* Write it to disk	*/
		return (ERROR);
	return (OK);
}

main()
{
	register int i, j;
	int	bps, spc, fats, dir, spd, spf, dat, chunk;
	int	sec, secs, res, drv1, drv2, sides, warn, same;
	long	memory, time;
	char	*buf1, *buf2;
	BPB	*bpb;

	char	*msg1 = "\r\n\tError reading!";
	char	*msg2 = "\r\n\tError writing!";


	Cconws("\033E\r\n\n\tFLEXCOPY\tby Moshe Braner");

ch1:	Cconws("\r\n\n\tEnter letter label of source drive: ");
	drv1 = Bconin(2);
	if (drv1>='a' && drv1<='p')
		drv1 += 'A'-'a';
	Bconout(2, drv1);
	if (drv1 < 'A' || drv1 > 'P')
		goto ch1;
	drv1 -= 'A';		/* 0=A, 1=B, ... */

ch2:	Cconws("\r\n\tEnter letter label of destination drive: ");
	drv2 = Bconin(2);
	if (drv2>='a' && drv2<='p')
		drv2 += 'A'-'a';
	Bconout(2, drv2);
	if (drv2 < 'A' || drv2 > 'P')
		goto ch2;
	drv2 -= 'A';		/* 0=A, 1=B, ... */

	if (drv2 == drv1) {
		Cconws("\r\n\tMust be different from source drive!");
		goto ch2;
	}
	same = 0;
	if (drv1+drv2 == 1) {
		Cconws("\r\n\tBoth the same physical drive (y/n)? ");
		i = Bconin(2);
		Bconout(2,i);
		if (i=='y' || i=='Y')
			same = 1;
	}

	/* get disk parameters from bios parameter block */

	Cconws("\r\n\tInsert source disk, hit any key: ");
	Bconin(2);
	bpb = (BPB *) Getbpb(drv1);
	bps = bpb->recsiz;		/* bytes per sector		*/
	spc = bpb->clsiz;		/* sectors per cluster		*/
	dir = bpb->rdlen;		/* length of dir in sectors	*/
	spf = bpb->fsiz;		/* sectors per FAT		*/
	dat = bpb->datrec;		/* no. of first data sector	*/
	spd = (bpb->numcl) * spc;	/* data sectors per disk	*/

	/* allocate RAM buffers */

	memory = (2*spf+dir)*bps;
	if ((buf1 = (char *)Malloc(memory)) <= 0)
		error("\r\n\tNot enough memory!");
	memory = Malloc(-1L);
	if (memory <  9216
	|| (buf2 = (char *)Malloc(memory)) <= 0)
		error("\r\n\tNot enough memory!");

	/* copy FATs and root directory from disk to RAM */

	secs = 2*spf + dir;		/* read FATs + dir		*/
	sec = dat - secs;
	if (Rwabs (READ, buf1, secs, sec, drv1) != OK)
		error(msg1);
	i = 3*spd;			/* >>>> 12-bit FAT entries	*/
	i = 3 + i/spc/2;
	while (buf1[--i] == 0);		/* search for last used sector	*/
	i *= spc*2;
	res = i/3;			/* no. of sectors left to read	*/
	if (res > spd)			/* safety check			*/
		res = spd;

	/* format destination disk */

	Cconws("\r\n\tInsert destination disk, hit any key: ");
	Bconin(2);
	warn = 1;
	if (drv2 < 2) {
		Cconws("\r\n\tFormat destination drive (y/n)? ");
		i = Bconin(2);
		Bconout(2, i);
		if (i=='y' || i=='Y') {
		    warn = 0;
		    Cconws("\r\n\n\t\tWARNING:");
		    Cconws("\r\n\tAbout to format disk in drive ");
		    Bconout(2,drv2+'A');
		    Cconws(",\r\n\terasing ALL data in that drive!");
		    Cconws("\r\n\n\t\tProceed (y/n)? ");
		    i = Bconin(2);
		    if (i=='y' || i=='Y') {
		        Cconws(
		        "\r\n\tSingle or double sided (s/d, default:s): ");
		        sides = Bconin(2);
		        if (sides=='\r')	sides='S';
		        Bconout(2,sides);
		        Cconws("\r\n\t[formatting...]");
		        if (format(drv2,sides) != OK)
		            error ("\r\n\tError while formatting!");
		    }
		}
	}

	/* compare source and destination disk */

	Cconws("\r\n\t");
	bpb = (BPB *) Getbpb(drv2);
	if (bpb->recsiz != bps
	||  bpb->clsiz  != spc
	||  bpb->rdlen  != dir
	||  bpb->fsiz   != spf
	||  bpb->datrec != dat)
		error("Disks incompatible!");
	if ((bpb->numcl)*spc < res)
		error("Not enough destination space!");

	/* write FATs and root directory to destination */

	if (warn) {
		Cconws("\r\n\t\tWARNING:");
		Cconws("\r\n\tWill overwrite ALL data in drive ");
		Bconout(2,drv2+'A');
		Cconws(".\r\n\n\t\tProceed (y/n)? ");
		i = Bconin(2);
		if (i!='y' && i!='Y')
			exit(0);
	}
	if (Rwabs (WRITE, buf1, secs, sec, drv2) != OK)
		error(msg2);
	
	/* copy rest of data */

	Cconws("\r\n\n\t[copying...]\r\n");

	chunk = memory/bps;	/* how many sectors fit in RAM */
	j = dat;
	i = res;
	while (i > 0) {
		if (chunk > i)
			chunk = i;
		if (same) {
		    Cconws("\r\n\tInsert source disk, hit any key: ");
		    Bconin(2);
		}
		if (Rwabs  (RREAD, buf2, chunk, j, drv1) != OK)
			error(msg1);
		if (same) {
		    Cconws("\r\n\tInsert destination disk, hit any key: ");
		    Bconin(2);
		}
		if (Rwabs (WWRITE, buf2, chunk, j, drv2) != OK)
			error(msg2);
		j += chunk;
		i -= chunk;
	}

	error("\r\n\n\tFLEXCOPY finished, no errors.");
}
