*
*   mono_clk.s:  Display current time on monochrome monitor 
*
*   use vertical blank interrupt as corrected from
*   Chap.2 'Tricks and Tips' ABAQUS 1986  CJPurcell 21AUG'86
*   normally used in monochrome autoboot eSTeClock; can be key-clicked alive.
*
*   Modified: 12-19-86 by John L. Stanley
*	Changed program to use gemtick vector (50hz) instead of 200hz clock
*	  This change was made to reduce the load of the clock on the system.
*	  The clock was sometimes taking too long to return which was
*	  screwing up the mouse/keyboard packets.
*	Removed (commented out) code to display shift-status.
*	Made clock roll-over to 1:00 after 12:59:59 instead of
*	  changing to 0:00 at 23:59:59 which better reflects how
*	  most people want to see the time.
*	Added am/pm indicator, removed seconds from display, and
*	  added a flashing colon to indicate the passing of seconds.
*

gemdos     equ   1
setexec    equ   5
bios       equ   13
xbios      equ   14
gettime    equ   23
super      equ   38   execute routine in supervisor mode
keep       equ   $31
* kbmode   equ   $e1a   keyboard shift (mode) indication w.
gemtick	   equ   $100	Gem timer interupt called every 20ms, (50hz)
_v_bas_ad  equ   $44e   screen address


*
*   system font ATARI ST
*
fontdat	  equ   76	ptr to font data
formwd    equ   80	status of next raster line in font
formhg    equ   82	number of raster lines per character
line1     equ   80	bytes per screen line
linea     equ   $a000	linea operator(pseudo instruction)for ATARI

start:
	move.l   4(sp),a0       calculate program size
	move.w   #$100,d6
	add.l    12(a0),d6
	bsr      init           program initialization
	clr      -(sp)
	move.l   d6,-(sp)       number bytes
	move     #keep,-(sp)  
	trap     #gemdos
init:
	dc.w     linea       *get linea pointers for initialization;d0,a0,a1,a2
	moveq    #2*4,d0         *font number ; of the three in array of reg a1
	lea      fontptr(pc),a3   *use smallest font for color; middle for mono
	move.l   $0(a1,d0),(a3)    *mark font pointer

	move     #gettime,-(sp)
	trap     #xbios
	addq.l   #2,sp
	move     d0,d7

	pea      sup_rout(pc)
	move     #super,-(sp)	* execute rest in supervisor mode
	trap     #xbios
	addq.l   #6,sp
	rts			* <-- jumps to sup_rout

sup_rout:			* convert time to hours:minutes:seconds
	move	 d7,d0
	and.w    #%11111,d0
	lsl      #1,d0		*seconds in binary
	move     d0,second

	move     d7,d0
	lsr      #5,d0
	and.w    #%111111,d0
	move     d0,minute

	move     d7,d0
	moveq    #11,d1
	lsr      d1,d0
	cmp.w    #13,d0		check for after 12:59:59 
	blt      am_hr
	move.w	 #$70,ampm	'p'
	sub	 #12,d0
am_hr:	move    d0,hour

	move.l   #25,time		; initialize microseconds counter

	pea      hz_int(pc)
	move     #$100,-(sp)		; replace gemtick interrupt vector
	move     #setexec,-(sp)
	trap     #bios
	addq.l   #8,sp

	move.l   d0,hz_save		; save previous gemtick vector
	rts				; done, exit...

hz_int:
	subq.l	#1,time
	ble	show		; half second yet? YES

	move.l	hz_save,-(sp)	; No, jump to previous gemtick vector
	rts

show:
	add.l	#25,time
	eor.w	#1,tictoc
	bne	toc		; only half second.. flash colon...

	addq.w   #1,second
	cmp.w    #60,second	; check seconds
	bne      show_time
	clr.w    second

	addq.w   #1,minute      next minute
	cmp.w    #60,minute     check minutes
	bne      show_time
	clr.w    minute

	addq.w   #1,hour
	cmp.w    #13,hour       check hours
	bne      show_time
	clr.w	 hour
	addq.w	 #1,hour
	eor.w	 #11,ampm	change 'a' into 'p' or 'p' into 'a'

show_time:
	move     #72,d6        cursor position;(adj. to allow .BAK op.in edit)
	move     hour,d0        get hour
	bsr      zwrtdec

*	bsr      wrtcol		; print ':'
	move	 #':',d0
	bsr      wrtchar

	move     minute,d0      get minute
	bsr      wrtdec

	move.w	ampm,d0
	bsr	wrtchar        output
	move.w	#'m',d0
	bsr     wrtchar        output

*        bsr      wrtcol		; print ':'
*
*        move     second,d0      get second
*        bsr      wrtdec

*                               
*       move     #0,a5          SHOW condition of CapsLock setting!
*       move     kbmode(a5),d0	this location found by investigation!
*       and      #%10000,d0
*       lsr      #4,d0          set indicator to up arrow, if present
*       move     #77,d6         cursor position of CapsLock signal
*       bsr      wrtchar        uses magic location for TOS in ROM 1986
*

int_exit:
	move.l   hz_save,-(sp)	; jump to previous gemtick vector
	rts

toc:
	move     #74,d6
	move	 #$20,d0	space character
	bsr      wrtchar
	move.l   hz_save,-(sp)	; jump to previous gemtick vector
	rts

zwrtdec:
	move.w	#$2f,d1        character ('0'-1)
zwrtdec1:
	addq.w  #1,d1
	sub.w   #10,d0
	bpl     wrtdec1

	add.w	#$3a,d0        add character ('9'+1)
	move.w  d0,-(sp)             
	move    d1,d0           
	cmp	#$30,d0		chack for leading '0'
	bne	notzero
	move.b	#$20,d0		Space character
notzero:
	bsr     wrtchar        output
	move.w  (sp)+,d0       unit
	bra     wrtchar        output

wrtdec:
	move.w	#$2f,d1        character ('0'-1)
wrtdec1:
	addq.w  #1,d1
	sub.w   #10,d0
	bpl     wrtdec1

	add.w	#$3a,d0        add character ('9'+1)
	move.w  d0,-(sp)             
	move    d1,d0           
	bsr     wrtchar        output
	move.w  (sp)+,d0       unit
	bra     wrtchar        output

*wrtcol:
**	move.w  #':',d0         ':'
*	move.w  coln,d0         ':'

*                               write characters to graphic ram
*                               where: d0 = character
*                                      d6 = cursor column
wrtchar:
	moveq   #0,d1
	move    d6,d1
	addq    #1,d6           move cursor to next column
	move.l  fontptr(pc),a3  get font pointer
	add.l   _v_bas_ad,d1    plus screen address
	move.l  d1,a4
	move.l  fontdat(a3),a0  font data pointer
	move    formwd(a3),d2   offset of next raster line in font
	move    formhg(a3),d7   foru height (number pf scan lines)
	subq    #1,d7
loop:
	move.b  $0(a0,d0),(a4)    onscreen raster line
	add     #line1,a4       pointer to next screen line
	add     d2,a0           pointer to next raster line in font
	dbra    d7,loop
	rts

fontptr ds.l     1
hz_save ds.l     1
second  ds.w     1
minute  ds.w     1
hour    ds.w     1
ampm	dc.w	 $0061		'a'
coln	dc.w	 $003a		':'
tictoc	dc.w	 0
time    ds.l     1
	.end
