PROGRAM Set_Info;

{
         *** Written in OSS Personal Pascal by Doug Harrison. ***
}

LABEL 1;
TYPE dta_buf    = ARRAY [0..21] OF INTEGER;     {22 words, 44 bytes}
     Path_Chars = PACKED ARRAY [1..80] OF CHAR;
     STR255     = STRING[255];
     buf_type   = ARRAY [1..1000] OF INTEGER;

VAR
   ps                                                             : STR255;
   del_name,c_low,c_med,c_high                                    : Path_Chars;
   dta_buffer                                                     : dta_buf;
   rez,attributes,result,i,chosen_rez,one,two,zero                : INTEGER;
   low_found,med_found,high_found                                 : BOOLEAN;

PROCEDURE Make_Path ( ps : STR255 ; VAR cs : Path_Chars );
   VAR i : INTEGER;
   BEGIN
       FOR i := 1 TO LENGTH ( ps ) DO
           cs[i] := ps[i];
       cs[ LENGTH ( ps )+1 ] := CHR ( 0 );
   END;

PROCEDURE Set_DTA ( VAR dta_buffer : dta_buf );
   GEMDOS ( $1A );

FUNCTION SFirst ( VAR file_name:Path_Chars ; attributes:INTEGER ) : INTEGER;
   GEMDOS ( $4E );

FUNCTION F_Create ( VAR name : Path_Chars ; attributes : INTEGER ) : INTEGER;
   GEMDOS ( $3C );

FUNCTION F_Open ( VAR name : Path_Chars ; mode : INTEGER ) : INTEGER;
   GEMDOS ( $3D );

FUNCTION F_Close ( handle : INTEGER ) : INTEGER;
   GEMDOS ( $3E );

FUNCTION F_Read ( handle : INTEGER; count : LONG_INTEGER;
                   VAR buf : buf_type ) : LONG_INTEGER;
   GEMDOS ( $3F );

FUNCTION F_Write ( handle : INTEGER; count : LONG_INTEGER;
                   VAR buf : buf_type ) : LONG_INTEGER;
   GEMDOS ( $40 );

FUNCTION F_Rename ( VAR new_name,old_name: Path_Chars; dummy:INTEGER ):INTEGER;
   GEMDOS ( $56 );

FUNCTION F_Delete ( VAR name : Path_Chars ) : INTEGER;
   GEMDOS ( $41 );

PROCEDURE BConOut ( device , c : INTEGER );
   BIOS ( $03 );

FUNCTION Get_Rez : INTEGER;
   XBIOS ( $04 );

PROCEDURE Char_Con ( c : INTEGER );
   CONST screen = 2;
   BEGIN
       bconout ( screen,c );
   END;

PROCEDURE Esc_Con ( c : CHAR );
   CONST esc = 27;
   BEGIN
       Char_Con ( esc );
       Char_Con ( ORD (c) );
   END;

PROCEDURE Blink_Cursor;
   BEGIN
       Esc_Con ( 'e' );
   END;

PROCEDURE UnBlink_Cursor;
   BEGIN
       Esc_Con ( 'f' );
   END;

FUNCTION F_Name_Exists ( file_name : Path_Chars; attributes: INTEGER ):BOOLEAN;
   VAR
       address    : LONG_INTEGER;
       found      : INTEGER;
   BEGIN
       found := sfirst ( file_name,attributes ); { sfirst needs dta_buffer }
       IF found = -33 THEN f_name_exists := FALSE
       ELSE f_name_exists := TRUE;
   END;

PROCEDURE Copy_File ( old_name,new_name : STR255 );

   { only called if your inf file(s) has been found to exist }

   VAR msw,size,bytes_read,bytes_written      : LONG_INTEGER;
       i,old_hdl,new_hdl,found                : INTEGER;
       f_buf                                  : buf_type;
       c_old_name,c_new_name                  : Path_Chars;
   BEGIN
       Make_Path ( old_name,c_old_name );
       Make_Path ( new_name,c_new_name );
       found := sfirst ( c_old_name,attributes ); { sfirst needs dta_buffer }
       old_hdl := F_Open ( c_old_name,0 );
       new_hdl := F_Create ( c_new_name,0 );
       {dta_buffer[13] and [14] hold the file size ( 2 words total ) }
       size := dta_buffer[13]*65536+dta_buffer[14];
       bytes_read := F_Read ( old_hdl,size,f_buf );
       bytes_written := F_Write ( new_hdl,bytes_read,f_buf );
       result := F_Close ( old_hdl );
       result := F_Close ( new_hdl );
   END;

PROCEDURE INITIALIZE;
   BEGIN
       rez := Get_Rez;
       attributes := 0;
       zero := 0;
       one := 1;
       two := 2;
       low_found := FALSE;
       med_found := FALSE;
       high_found := FALSE;
       { needn't be more specific since if no pathname is specified, GEMDOS
         assumes you're referring to the drive you are currently accessing
         and the root directory of that drive }
       Make_Path ( 'LOW.INF' , c_low );
       Make_Path ( 'MED.INF' , c_med );
       Make_Path ( 'HIGH.INF' , c_high );
       Set_DTA ( dta_buffer );  { for sfirst }
       Blink_Cursor;
       writeln ( ' ' );
   END;

BEGIN

    initialize;

    IF rez = 0 THEN BEGIN { color monitor always starts off in low rez }
       low_found := F_Name_Exists ( c_low,attributes );
       med_found := F_Name_Exists ( c_med,attributes );
       IF ( NOT low_found ) AND ( NOT med_found ) THEN BEGIN
          writeln ( 'INFO files not found.' );
          writeln ( 'Press a key to boot...' );
          WHILE NOT KeyPress DO;
          GOTO 1;
       END;
       IF ( low_found ) AND ( med_found ) THEN BEGIN
          writeln ( 'Which resolution?' );
          writeln ( '   1. Low' );
          writeln ( '   2. Medium' );
          chosen_rez := 0;
          WHILE ( chosen_rez <> 1 ) AND ( chosen_rez <> 2 ) DO
             readln ( chosen_rez );
          IF chosen_rez = 2 THEN low_found := FALSE
          ELSE med_found := FALSE;
       END;
    END
    ELSE BEGIN
       high_found := F_Name_Exists ( c_high,attributes );
       IF NOT high_found THEN BEGIN
          writeln ( 'INFO file not found. Press a key to boot...' );
          WHILE NOT KeyPress DO;
          GOTO 1;
       END;
    END;

    ps := 'DESKTOP.INF';
    Make_Path ( ps,del_name );
    result := F_Delete ( del_name );
    IF ( result < 0 ) AND ( result <> -33 ) THEN BEGIN
       writeln ( 'An error occurred. Check disk write' );
       writeln ( 'protection, DESKTOP.INF read/write' );
       writeln ( 'status, etc. Press a key to boot...' );
       WHILE NOT KeyPress DO;
       GOTO 1;
    END;

    IF low_found THEN
       copy_file ( 'LOW.INF','DESKTOP.INF' );
    IF med_found THEN
       copy_file ( 'MED.INF','DESKTOP.INF' );
    IF high_found THEN
       copy_file ( 'HIGH.INF','DESKTOP.INF' );

1:  UnBlink_Cursor;
END.
